@@ -7,10 +7,11 @@
 //! domains and walls, for example wall bounded flows in fluid mechanics.
 //!
 //! See [`ortho::Chebyshev`]
+#![allow(clippy::module_name_repetitions)]
 mod composite;
 mod composite_stencil;
+mod dmsuite;
 mod linalg;
 mod ortho;
-use crate::FloatNum;
 pub use composite::CompositeChebyshev;
 pub use ortho::Chebyshev;
