@@ -7,43 +7,27 @@ use crate::FourierC2c;
 use crate::FourierR2c;
 use ndarray::prelude::*;
 
-// pub trait SuperBase<F, R, S>:
-//     BaseBasics<F>
-//     + Transform<R, S>
-//     + TransformPar<R, S>
-//     + FromOrtho<S>
-//     + FromOrthoPar<S>
-//     + Differentiate<S>
-//     + LaplacianInverse<F>
-// {
-// }
-
-// impl<T, F, R, S> SuperBase<F, R, S> for T where
-//     T: BaseBasics<F>
-//         + Transform<R, S>
-//         + TransformPar<R, S>
-//         + FromOrtho<S>
-//         + FromOrthoPar<S>
-//         + Differentiate<S>
-//         + LaplacianInverse<F>
-// {
-// }
-
 /// Some basic  traits
 #[enum_dispatch]
-pub trait Basics<T> {
+pub trait Basics<T>: BaseSize {
     /// Coordinates in physical space
     fn coords(&self) -> &Array1<T>;
-    /// Size in physical space
-    fn len_phys(&self) -> usize;
-    /// Size in spectral space
-    fn len_spec(&self) -> usize;
     /// Return mass matrix
     fn mass(&self) -> Array2<T>;
     /// Return kind of transform
     fn get_transform_kind(&self) -> &TransformKind;
 }
 
+#[enum_dispatch]
+pub trait BaseSize {
+    /// Size in physical space
+    fn len_phys(&self) -> usize;
+    /// Size in spectral space
+    fn len_spec(&self) -> usize;
+    /// Size of orthogonal space
+    fn len_orth(&self) -> usize;
+}
+
 /// Transform from physical to spectral space and vice versa.
 ///
 /// The associated types *Physical* and *Spectral* refer
@@ -77,11 +61,7 @@ pub trait Transform {
     /// let output = cheby.forward(&mut input, 0);
     /// approx_eq(&output, &array![2.5, 1.33333333, 0. , 0.16666667]);
     /// ```
-    fn forward<S, D>(
-        &mut self,
-        input: &mut ArrayBase<S, D>,
-        axis: usize,
-    ) -> Array<Self::Spectral, D>
+    fn forward<S, D>(&mut self, input: &ArrayBase<S, D>, axis: usize) -> Array<Self::Spectral, D>
     where
         S: ndarray::Data<Elem = Self::Physical>,
         D: Dimension;
@@ -92,7 +72,7 @@ pub trait Transform {
     /// be supplied instead of being created.
     fn forward_inplace<S1, S2, D>(
         &mut self,
-        input: &mut ArrayBase<S1, D>,
+        input: &ArrayBase<S1, D>,
         output: &mut ArrayBase<S2, D>,
         axis: usize,
     ) where
@@ -119,11 +99,7 @@ pub trait Transform {
     /// let output = cheby.backward(&mut input, 0);
     /// approx_eq(&output, &array![-2. ,  2.5, -3.5, 10.]);
     /// ```
-    fn backward<S, D>(
-        &mut self,
-        input: &mut ArrayBase<S, D>,
-        axis: usize,
-    ) -> Array<Self::Physical, D>
+    fn backward<S, D>(&mut self, input: &ArrayBase<S, D>, axis: usize) -> Array<Self::Physical, D>
     where
         S: ndarray::Data<Elem = Self::Spectral>,
         D: Dimension;
@@ -134,7 +110,7 @@ pub trait Transform {
     /// be supplied instead of being created.
     fn backward_inplace<S1, S2, D>(
         &mut self,
-        input: &mut ArrayBase<S1, D>,
+        input: &ArrayBase<S1, D>,
         output: &mut ArrayBase<S2, D>,
         axis: usize,
     ) where
@@ -179,7 +155,7 @@ pub trait TransformPar {
     /// ```
     fn forward_par<S, D>(
         &mut self,
-        input: &mut ArrayBase<S, D>,
+        input: &ArrayBase<S, D>,
         axis: usize,
     ) -> Array<Self::Spectral, D>
     where
@@ -192,7 +168,7 @@ pub trait TransformPar {
     /// be supplied instead of being created.
     fn forward_inplace_par<S1, S2, D>(
         &mut self,
-        input: &mut ArrayBase<S1, D>,
+        input: &ArrayBase<S1, D>,
         output: &mut ArrayBase<S2, D>,
         axis: usize,
     ) where
@@ -221,7 +197,7 @@ pub trait TransformPar {
     /// ```
     fn backward_par<S, D>(
         &mut self,
-        input: &mut ArrayBase<S, D>,
+        input: &ArrayBase<S, D>,
         axis: usize,
     ) -> Array<Self::Physical, D>
     where
@@ -234,7 +210,7 @@ pub trait TransformPar {
     /// be supplied instead of being created.
     fn backward_inplace_par<S1, S2, D>(
         &mut self,
-        input: &mut ArrayBase<S1, D>,
+        input: &ArrayBase<S1, D>,
         output: &mut ArrayBase<S2, D>,
         axis: usize,
     ) where
@@ -303,9 +279,9 @@ pub trait LaplacianInverse<T> {
     fn laplace_inv_eye(&self) -> Array2<T>;
 }
 
-/// Define transformation from and to orthonormal space.
+/// Define transformation from and to orthogonal space.
 ///
-/// If the space is already the orthonormal (parent)
+/// If the space is already the orthogonal (parent)
 /// space, it copies and returns the input.
 #[enum_dispatch]
 pub trait FromOrtho<T> {
@@ -388,7 +364,7 @@ pub trait FromOrtho<T> {
         D: Dimension;
 }
 
-/// Define transformation from and to orthonormal space.
+/// Define transformation from and to orthogonal space.
 /// (Parallel version which uses ndarrays `par_for_each` iterator)
 #[enum_dispatch]
 pub trait FromOrthoPar<T> {
